@@ -34,7 +34,9 @@ and then returned to the caller.
 import collections
 import itertools
 
+import jsonpatch
 from keystoneauth1 import adapter
+import munch
 from requests import structures
 
 from openstack import _log
@@ -313,6 +315,8 @@ class Resource(object):
     update_method = "PUT"
     #: Method for creating a resource (POST, PUT)
     create_method = "POST"
+    #: Whether commit uses JSON patch format.
+    commit_jsonpatch = False
 
     #: Do calls for this resource require an id
     requires_id = True
@@ -321,6 +325,8 @@ class Resource(object):
     #: Is this a detailed version of another Resource
     detail_for = None
 
+    _original_body = None
+
     def __init__(self, _synchronized=False, **attrs):
         """The base resource
 
@@ -343,6 +349,17 @@ class Resource(object):
                                          synchronized=_synchronized)
         self._uri = _ComponentManager(attributes=uri,
                                       synchronized=_synchronized)
+        if self.commit_jsonpatch:
+            # We need the original body to compare against
+            if _synchronized:
+                self._original_body = self._body.attributes.copy()
+            elif self.id:
+                # Never record ID as dirty.
+                self._original_body = {
+                    self._alternate_id() or 'id': self.id
+                }
+            else:
+                self._original_body = {}
 
     def __repr__(self):
         pairs = ["%s=%s" % (k, v) for k, v in dict(itertools.chain(
@@ -573,7 +590,8 @@ class Resource(object):
         """
         return cls(_synchronized=synchronized, **obj)
 
-    def to_dict(self, body=True, headers=True, ignore_none=False):
+    def to_dict(self, body=True, headers=True, ignore_none=False,
+                original_names=False):
         """Return a dictionary of this resource's contents
 
         :param bool body: Include the :class:`~openstack.resource.Body`
@@ -583,6 +601,8 @@ class Resource(object):
         :param bool ignore_none: When True, exclude key/value pairs where
                                  the value is None. This will exclude
                                  attributes that the server hasn't returned.
+        :param bool original_names: When True, use attribute names as they
+                                    were received from the server.
 
         :return: A dictionary of key/value pairs where keys are named
                  as they exist as attributes of this class.
@@ -608,12 +628,16 @@ class Resource(object):
         # Since we're looking at class definitions we need to include
         # subclasses, so check the whole MRO.
         for klass in self.__class__.__mro__:
-            for key, value in klass.__dict__.items():
-                if isinstance(value, components):
+            for attr, component in klass.__dict__.items():
+                if isinstance(component, components):
+                    if original_names:
+                        key = component.name
+                    else:
+                        key = attr
                     # Make sure base classes don't end up overwriting
                     # mappings we've found previously in subclasses.
                     if key not in mapping:
-                        value = getattr(self, key, None)
+                        value = getattr(self, attr, None)
                         if ignore_none and value is None:
                             continue
                         if isinstance(value, Resource):
@@ -629,7 +653,13 @@ class Resource(object):
 
         return mapping
 
-    def _prepare_request(self, requires_id=None, prepend_key=False):
+    def _to_munch(self):
+        """Convert this resource into a Munch compatible with shade."""
+        return munch.Munch(self.to_dict(body=True, headers=False,
+                                        original_names=True))
+
+    def _prepare_request(self, requires_id=None, prepend_key=False,
+                         patch=False):
         """Prepare a request to be sent to the server
 
         Create operations don't require an ID, but all others do,
@@ -638,6 +668,7 @@ class Resource(object):
         their bodies to be contained within an dict -- if the
         instance contains a resource_key and prepend_key=True,
         the body will be wrapped in a dict with that key.
+        If patch=True, a JSON patch is prepared instead of the full body.
 
         Return a _Request object that contains the constructed URI
         as well a body and headers that are ready to send.
@@ -646,9 +677,13 @@ class Resource(object):
         if requires_id is None:
             requires_id = self.requires_id
 
-        body = self._body.dirty
-        if prepend_key and self.resource_key is not None:
-            body = {self.resource_key: body}
+        if patch:
+            new = self._body.attributes
+            body = jsonpatch.make_patch(self._original_body, new).patch
+        else:
+            body = self._body.dirty
+            if prepend_key and self.resource_key is not None:
+                body = {self.resource_key: body}
 
         # TODO(mordred) Ensure headers have string values better than this
         headers = {}
@@ -688,6 +723,9 @@ class Resource(object):
             body = self._consume_body_attrs(body)
             self._body.attributes.update(body)
             self._body.clean()
+            if self.commit_jsonpatch:
+                # We need the original body to compare against
+                self._original_body = body.copy()
 
         headers = self._consume_header_attrs(response.headers)
         self._header.attributes.update(headers)
@@ -819,7 +857,13 @@ class Resource(object):
         if not self.allow_update:
             raise exceptions.MethodNotSupported(self, "update")
 
-        request = self._prepare_request(prepend_key=prepend_key)
+        # Avoid providing patch unconditionally to avoid breaking subclasses
+        # without it.
+        kwargs = {}
+        if self.commit_jsonpatch:
+            kwargs['patch'] = True
+
+        request = self._prepare_request(prepend_key=prepend_key, **kwargs)
         session = self._get_session(session)
 
         if self.update_method == 'PATCH':
